﻿using System.Collections.Generic;
using Ueberpruefung;

namespace Eingabeverwaltung.Metamodell
{
	public abstract class Rule : Element
	{
		public readonly string name;
		public readonly Equation eq;
		public Rule(string name, Equation equation) 
		{
			this.name = name;
			eq = equation;
		}

		/// <summary>
		/// @pre: variable must be in eq.variables
		/// @post: result is given: if variable is fixed
		/// </summary>
		/// <param name="variable"></param>
		/// <returns> true if variable is fixed, else false </returns>
		public abstract bool IsFixed(Variable variable);

		/// <summary>
		/// For every variable in equation: Rule.subtype.IsFixed(variable).
		/// </summary>
		/// <returns>List of all variables of equation, for which subtype says its fixed.</returns>
		public List<Variable> GetFixedVars()
        {
			List<Variable> fixedVars = new List<Variable>();
			foreach (Variable var in eq.variables)
				if (IsFixed(var)) fixedVars.Add(var);
			return fixedVars;
		}

        public override string ToString()
        {
			string res = name + ": ";
			List<Variable> fixedVars = GetFixedVars();
			if (fixedVars.Count > 0) res += "fixed ";
			foreach (Variable var in fixedVars)
				res += var.ToString();
			if (fixedVars.Count > 0) res += ": ";
			if (eq.variables.Count - fixedVars.Count > 0)
			{
				res += "forall ";
				List<Variable> allQVars = new List<Variable>(eq.variables);
				foreach (Variable var in fixedVars)
					allQVars.Remove(var);
				foreach (Variable var in allQVars)
					res += var.ToString();
				res += ": ";
			}
			res += eq.leftTree.ToString() + " = " + eq.rightTree.ToString() + "\r\n";
			return res;
		}
    }
}
