﻿using Eingabeverwaltung.Metamodell;
using System;
using System.Collections.Generic;
using System.Collections.Immutable;
using System.IO;

namespace Testverwaltung
{
    public class Testverwalter
    {
        private Container con;
        private ImmutableDictionary<string, Action> TestManager;
        public Testverwalter(Container con)
        {
            this.con = con;
        }

        public void verwalteTests()
        {
            string ausgabe = "Welche Tests sollen gestartet werden?";
            TestManager = new Dictionary<string, Action>
            {
                ["manuell"] = Manual,
                ["all auto"] = TestAll,
                ["Transformation"] = TestTransformation,
                ["Lemma"] = TestLemma,
                ["Induktion"] = TestInduction,
                ["?"] = giveHelp,
                ["exit"] = Exit_Tests,
            }.ToImmutableDictionary();
            Console.WriteLine(ausgabe);
            string eingabeArt = Console.ReadLine();
            if (eingabeArt == "manuell" && con == null)
            { 
                Console.WriteLine("Cannot start manual Test. First fill the Container!");
                verwalteTests(); return;
            }
            try
            {                
                TestManager[eingabeArt]();
            }
            catch (KeyNotFoundException) { Console.WriteLine("Command not found. Type ? for help."); verwalteTests(); }
        }

        /// <summary>
        /// Starts manual Tests.
        /// </summary>
        private void Manual() => new ManualTests(con);

        private void giveHelp()
        {
            foreach (string key in TestManager.Keys)
                Console.WriteLine(key);
            verwalteTests();
        }

        /// <summary>
        /// Does nothing in order to let the Eingabe exit.
        /// </summary>
        private void Exit_Tests() { }
        /// <summary>
        /// Starts All Tests of Errorfinding.
        /// </summary>
        private void TestAll() => Test(new List<Test>() { new TransformationTest(), new LemmaTest(), new InductionTest() });
        /// <summary>
        /// Starts Test of Errorfinding for Transformation.
        /// </summary>
        private void TestTransformation() => Test(new List<Test>() { new TransformationTest() }); 
        /// <summary>
        /// Starts Test of Errorfinding for Lemma.
        /// </summary>
        private void TestLemma() => Test(new List<Test>() { new LemmaTest() });
        /// <summary>
        /// Starts Test of Errorfinding for Induction
        /// </summary>
        private void TestInduction() => Test(new List<Test>() { new InductionTest() });

        private void Test(List<Test> tests)
        {
            string resultString = ""; bool result = true; int numTests = 0; DateTime time = DateTime.Now;
            foreach (Test test in tests)
            {
                result = result && test.run();
                resultString += test.getResultString();
                numTests += test.getNumTests();
            }
            File.WriteAllText(fileName(), startText(numTests,time) + resultString);
            Console.WriteLine("Testergebnis: bestanden = " + result + ". Mehr Details sind in txt Datei.");
        }

        private string startText(int numTests, DateTime startTime)
        {
            return "Test of BaumTransformationBewerter according:\r\n" +
                   "Finding of Errors\r\n" +
                   "Number of Tests: " + numTests + "\r\n" +
                   "Time started: " + startTime + "\r\n" +
                   "Run Time "+((DateTime.Now-startTime))+"\r\n\r\n";
        }

        private string fileName() => "Test Ergebnisse.txt";
    }
}
