﻿using Eingabeverwaltung.Metamodell;
using System;
using System.Collections.Generic;

namespace Ueberpruefung.CheckInduction
{
    /// <summary>
    /// checks error 34
    /// <br></br>
    /// sets constructor and buildingParamaterVariable of ind cases if context is Induction
    /// </summary>
    public class CheckIndPartEquation : ElementChecker
    {
        private readonly Induction parent;
        public CheckIndPartEquation(Induction parent) : base() { this.parent = parent; }
        public override bool check(Element e)
        {
            if (parent.indVar == null) return true;
            if (e is InductionHypothesis ih)
            {
                CheckHelper.getSubsAndRenamings(parent.eq, ih.eq, out List<Substitution> subs, out List<Substitution> renamings);
                bool result = true;
                string errOutput = "";
                // check sub
                if (subs == null || subs.Count != 0)
                {
                    result = false;
                    // find any buildingParameterVariable in one of the Inductioncases, if existing
                    Variable bPV = null;
                    foreach (InductionCase ic in parent.indCases)
                        if (ic.BuildingParameterVariables != null) { bPV = ic.BuildingParameterVariables[0]; break; }
                    if (bPV != null)
                        errOutput += constructCorrectedEquation(parent.eq, parent.fixedVars, new Substitution(parent.indVar, new TreeVariable(bPV)));
                }
                else// check renamings: the name should not change, except for indVar
                    foreach (Substitution sub in renamings)
                        if (sub.variable != parent.indVar)
                            result &= sub.variable.name == ((TreeVariable)sub.tree).name;
                if (!result) ih.errors.Add(new Error(34, errOutput));
                return result;
            }
            else if (e is InductionCase ic)
            {
                CheckHelper.getSubsAndRenamings(parent.eq, ic.eq, out List<Substitution> subs, out List<Substitution> renamings); 
                // check sub, only 1 real substitution allowed
                bool result = subs != null && subs.Count == 1; string errOutput = "";
                if (result && subs[0].tree is ComposedTree cTree)
                {                    
                    if (cTree.operation is Constructor c) // if its not constructor, then its job of 33
                    {                        
                        {// 1. safe constructor and buildingParameterVariable
                            ic.Constructor = c;
                            if (c is BuildingConstructor bc)
                            {
                                ic.BuildingParameterVariables = new List<Variable>();
                                bool variableIsOnBuildingParameter = true;
                                foreach (int buildingParameter in bc.buildingParameters)
                                    if (cTree.children[buildingParameter] is TreeVariable tv)
                                        ic.BuildingParameterVariables.Add(tv.variable);
                                    else // instead of inductionvariable an inner Tree or constant is used
                                        variableIsOnBuildingParameter = false;
                                if (variableIsOnBuildingParameter && ic.BuildingParameterVariables.Count == 0)
                                    throw new Exception("Found building constructor " + bc.name + " without building parameters!");
                                result &= variableIsOnBuildingParameter;
                            }
                        }
                        // 2. all children must be variable
                        List<string> childVars = new List<string>();
                        foreach (Tree child in cTree.children) 
                            if (child is TreeVariable tv) childVars.Add(tv.variable.name);
                            else result = false;
                        // 3. get all variables from parent, except ind var
                        List<string> indVars = new List<string>();
                        foreach (Variable var in parent.eq.variables)
                            indVars.Add(var.name);
                        indVars.Remove(parent.indVar.name);
                        // 4. the children's variables must be new for the equation (except indVar)
                        for (int i = 0; i < childVars.Count; i++)
                            if (!indVars.Contains(childVars[i])) { childVars.RemoveAt(i); i--; }
                        result &= childVars.Count == 0;
                        // 5. check renamings: the name should not change
                        foreach (Substitution sub in renamings)
                            result &= sub.variable.name == ((TreeVariable)sub.tree).name;
                        // 6. only ind var is allowed to be substituted by composed tree
                        result &= subs[0].variable == parent.indVar;
                        // 7. build corrected constructor for error output
                        if (!result)
                        {
                            if (parent.indVar.sort == cTree.sort)
                            {
                                List<Tree> children = new List<Tree>();
                                for (int i = 0; i < cTree.children.Count; i++)
                                    if (cTree.children[i] is TreeVariable tv && !childVars.Contains(tv.name)) children.Add(tv);
                                    else children.Add(new TreeVariable(new Variable("Y-" + i, cTree.children[i].sort)));
                                ComposedTree correctConstructor = new ComposedTree(cTree.operation, children);
                                errOutput = constructCorrectedEquation(parent.eq, parent.fixedVars, new Substitution(parent.indVar, correctConstructor));
                            }
                        }
                    }
                }
                if (!result) ic.errors.Add(new Error(34, errOutput));
                return result;
            }
            else throw new InvalidOperationException();
        }

        /// <summary>
        /// @pre: if sub.tree is ComposedTree :  all children must be TreeVariable
        /// </summary>
        /// <param name="fromEq"></param>
        /// <param name="fixedVars"></param>
        /// <param name="sub"></param>
        /// <param name="bPV">buildingParameterVariable</param>
        /// <returns></returns>
        private string constructCorrectedEquation(Equation fromEq, List<Variable> fromFixedVars, Substitution sub)
        {
            Tree leftTree = CheckHelper.substitute(fromEq.leftTree.Copy(), sub);
            Tree rightTree = CheckHelper.substitute(fromEq.rightTree.Copy(), sub);
            List<Variable> variables = new List<Variable>(fromEq.variables); variables.Remove(sub.variable);
            Equation eq = new Equation(leftTree, rightTree, variables);
            List<Variable> fixedVars = new List<Variable>(fromFixedVars);
            if (sub.tree is ComposedTree cTree)
            {   // constructing for indCase
                List<Variable> constructorVariables = new List<Variable>();
                foreach (Tree child in cTree.children)
                    if (child is TreeVariable tv)
                        constructorVariables.Add(tv.variable);
                variables.AddRange(constructorVariables);
                if (cTree.operation is BuildingConstructor bc)
                    foreach(int buildingParameter in bc.buildingParameters)
                        if (cTree.children[buildingParameter] is TreeVariable tv)
                            fixedVars.Add(tv.variable);
            }
            else if (sub.tree is TreeVariable tr)
            {   // constructing for indHypothesis
                variables.Add(tr.variable);
                fixedVars.Add(tr.variable);
            }
            string result = new TaskLemma("", eq, fixedVars).ToString();
            return result.Remove(result.Length - 2); // remove new line character at the end
        }
    }
}
