﻿using Eingabeverwaltung.Metamodell;
using System;
using System.Collections.Generic;
using System.Linq;

namespace Ueberpruefung.CheckTransformation
{
    public class CheckTask_Satisfied : ElementChecker
    {
        public CheckTask_Satisfied() { }

        public override bool check(Element e)
        {
            if (e is Container con)
                if (con.Proof.proof is Transformation tr)
                    return check(con.Task.task.eq, tr);
                else if (con.Proof.proof is Induction) return true; // is true by creation of Induction
                else throw new Exception("Found a Singleproof that is either Transformation nor Induction");
            else if (e is Lemma l)
                if (l.proof is Transformation tr)
                    return check(l.eq, tr);
                else if (l.proof is Induction) return true; // is true by creation of Induction
                else throw new Exception("Found a Singleproof that is either Transformation nor Induction");
            else if (e is InductionCase ic) return check(ic.eq, ic.proof);
            else throw new InvalidOperationException();
        }

        private bool check(Equation task, Transformation tr)
        {
            List<Error> errors = check(task, new Equation(tr.startTree, tr.transformationSteps.Last().resultTree, null));
            if (errors == null) return true;
            tr.errors.AddRange(errors);
            return false;
        }

        private List<Error> check(Equation task ,Equation proofResult)
        {
            List<Error> result = new List<Error>();
            bool startL = proofResult.leftTree.Equals(task.leftTree);
            bool startR = proofResult.leftTree.Equals(task.rightTree);
            bool endL = proofResult.rightTree.Equals(task.leftTree);
            bool endR = proofResult.rightTree.Equals(task.rightTree);
            if (startL && endR || startR && endL) result = null;
            else
            {
                if (!startL && !startR) result.Add(new Error(1));
                if (!endL && !endR || startL && !endR || startR && !endL) result.Add(new Error(2));
            }
            return result;
        }
    }
}
